% FUNDMATRIX7 - computes fundamental matrix from 7 or more points
% 
%Author Unknown%
%Modified by Aveek Brahmachari%

function [F1, F2, F3] = fundmatrix7(varargin)
    
    [x11, x21, npts] = checkargs(varargin(:));
       
    % Normalise each set of points so that the origin 
    % is at centroid and mean distance from origin is sqrt(2). 
    % normalise2dpts also ensures the scale parameter is 1.
     [x1, T1] = normalise2dpts(x11);
     [x2, T2] = normalise2dpts(x21);
    
    % Build the constraint matrix
    A = [x2(1,:)'.*x1(1,:)'   x2(1,:)'.*x1(2,:)'  x2(1,:)' ...
         x2(2,:)'.*x1(1,:)'   x2(2,:)'.*x1(2,:)'  x2(2,:)' ...
         x1(1,:)'             x1(2,:)'            ones(npts,1) ];       

    [~,~,V] = svd(A'*A, 0);
    G = reshape(V(:,8), 3, 3)';
    H = reshape(V(:,9), 3, 3)';

    % undo the scaling
     G = T2'*G*T1;  G = G/norm(G,2);
     H = T2'*H*T1;  H = H/norm(H,2);

    G11=G(1,1); G12=G(1,2); G13=G(1,3);
    G21=G(2,1); G22=G(2,2); G23=G(2,3);
    G31=G(3,1); G32=G(3,2); G33=G(3,3);

    H11=H(1,1); H12=H(1,2); H13=H(1,3);
    H21=H(2,1); H22=H(2,2); H23=H(2,3);
    H31=H(3,1); H32=H(3,2); H33=H(3,3);

    % the fundamental matrix, F, that we are after is a linear combination
    % of G and H, i.e. F = alpha*G + (1-alpha)*H.  We impose the det(F)=0
    % constraint to find alpha.  The condition det(F)=0 gives a polynomial of degree
    % 3 for alpha.  So we would either get one real solution for alpha or three
    % real solutions for alpha.  We need to deal with the latter case later.
    % Below are the coefficients of the polynomial, obtained from Matlab's
    % symbolic maths toolbox.

    coeffs = zeros(4,1);
    % coeffcient of the alpha^3 term
    coeffs(1) = -H31*H12*H23-H11*H22*H33-G11*G22*H33+ ...
       H11*H23*H32-G11*H23*H32-H21*H13*H32+H21*H12*H33+ ...
       H31*H13*H22+G11*G23*H32+H11*G22*H33+G11*G22*G33- ...
       G11*H22*G33-G11*G23*G32-H11*G22*G33+G11*H23*G32+ ...
       G11*H22*H33+H11*H22*G33+H11*G23*G32-H11*G23*H32- ...
       H11*H23*G32-G21*G12*G33+G21*G12*H33+G21*H12*G33- ...
       G21*H12*H33+G21*G13*G32-G21*G13*H32-G21*H13*G32+ ...
       G21*H13*H32+H21*G12*G33-H21*G12*H33-H21*H12*G33- ...
       H21*G13*G32+H21*G13*H32+H21*H13*G32+G31*G12*G23- ...
       G31*G12*H23-G31*H12*G23+G31*H12*H23-G31*G13*G22+ ...
       G31*G13*H22+G31*H13*G22-G31*H13*H22-H31*G12*G23+ ...
       H31*G12*H23+H31*H12*G23+H31*G13*G22-H31*G13*H22- ...
       H31*H13*G22;
    % coefficient of the alpha^2 term
    coeffs(2) = 3*H31*H12*H23+3*H11*H22*H33+ ...
       G11*G22*H33-3*H11*H23*H32+2*G11*H23*H32+ ...
       3*H21*H13*H32-3*H21*H12*H33-3*H31*H13*H22- ...
       G11*G23*H32-2*H11*G22*H33+G11*H22*G33+ ...
       H11*G22*G33-G11*H23*G32-2*G11*H22*H33- ...
       2*H11*H22*G33-H11*G23*G32+2*H11*G23*H32+ ...
       2*H11*H23*G32-G21*G12*H33-G21*H12*G33+ ...
       2*G21*H12*H33+G21*G13*H32+G21*H13*G32- ...
       2*G21*H13*H32-H21*G12*G33+2*H21*G12*H33+ ...
       2*H21*H12*G33+H21*G13*G32-2*H21*G13*H32- ...
       2*H21*H13*G32+G31*G12*H23+G31*H12*G23- ...
       2*G31*H12*H23-G31*G13*H22-G31*H13*G22+ ...
       2*G31*H13*H22+H31*G12*G23-2*H31*G12*H23- ...
       2*H31*H12*G23-H31*G13*G22+2*H31*G13*H22+ ...
       2*H31*H13*G22;
    % coefficient of the alpha term
    coeffs(3) = G31*H12*H23+H11*H22*G33- ...
       G21*H12*H33-H31*G13*H22+G11*H22*H33+ ...
       3*H31*H13*H22-H11*G23*H32-3*H21*H13*H32- ...
       G31*H13*H22+3*H11*H23*H32+G21*H13*H32- ...
       3*H31*H12*H23-G11*H23*H32+3*H21*H12*H33- ...
       H31*H13*G22-3*H11*H22*H33+H21*H13*G32+ ...
       H21*G13*H32-H21*H12*G33-H21*G12*H33+ ...
       H31*H12*G23+H31*G12*H23-H11*H23*G32+ ...
       H11*G22*H33;
    % coefficient of the constant term
    coeffs(4) = H31*H12*H23+H11*H22*H33- ...
       H11*H23*H32+H21*H13*H32-H21*H12*H33-H31*H13*H22;

    alphas = roots(coeffs);
    no_solns = 0;  % number of real solutions
    for i=1:3
       if isreal(alphas(i))
          no_solns = no_solns + 1;
          F{no_solns} = alphas(i)*G + (1-alphas(i))*H;
          F{no_solns} = F{no_solns} / norm(F{no_solns},2);
       end
    end

    Fmat = cell2mat(F);
    F1 = eye(3);
    F2 = eye(3);
    F3 = eye(3);

    if (no_solns==3)
        F1 = Fmat(1:3,1:3);
        F2 = Fmat(1:3,4:6);
        F3 = Fmat(1:3,7:9);
    end

    if (no_solns==2)
        F1 = Fmat(1:3,1:3);
        F2 = Fmat(1:3,4:6);
    end

    if (no_solns==1)
        F1 = Fmat;
    end
end  
%--------------------------------------------------------------------------
% Function to check argument values and set defaults

function [x1, x2, npts] = checkargs(arg)
    
    if length(arg) == 2
        x1 = arg{1};
        x2 = arg{2};
        if ~all(size(x1)==size(x2))
            error('x1 and x2 must have the same size');
        elseif size(x1,1) ~= 3
            error('x1 and x2 must be 3xN');
        end
        
    elseif length(arg) == 1
        if size(arg{1},1) ~= 6
            error('Single argument x must be 6xN');
        else
            x1 = arg{1}(1:3,:);
            x2 = arg{1}(4:6,:);
        end
    else
        error('Wrong number of arguments supplied');
    end
      
    npts = size(x1,2);
    if npts < 7
        error('At least 7 points are needed to compute the fundamental matrix');
    end 
end

function [newpts, T] = normalise2dpts(pts)

    if size(pts,1) ~= 3
        error('pts must be 3xN');
    end
    
    % Find the indices of the points that are not at infinity
    finiteind = find(abs(pts(3,:)) > eps);
    
    if length(finiteind) ~= size(pts,2)
        warning('Some points are at infinity');
    end
    
    % For the finite points ensure homogeneous coords have scale of 1
    pts(1,finiteind) = pts(1,finiteind)./pts(3,finiteind);
    pts(2,finiteind) = pts(2,finiteind)./pts(3,finiteind);
    pts(3,finiteind) = 1;
    
    c = mean(pts(1:2,finiteind)')';            % Centroid of finite points
    newp(1,finiteind) = pts(1,finiteind)-c(1); % Shift origin to centroid.
    newp(2,finiteind) = pts(2,finiteind)-c(2);
    
    meandist = mean(sqrt(newp(1,finiteind).^2 + newp(2,finiteind).^2));
    
    scale = sqrt(2)/meandist;
    
    T = [scale   0   -scale*c(1)
         0     scale -scale*c(2)
         0       0      1      ];
    
    newpts = T*pts;
end